//
// derived from AVR TM1638 "Library" v1.01
//
#ifndef HC595AE_H
#define HC595AE_H

#include <inttypes.h>

// ----------------------------------------------------------------------------
//
// PIN / PORT configuration
//
// NOTE: All three pins used must be bits on the same PORT register (ex. PORTB)
//
// ----------------------------------------------------------------------------
// PORTC
// PC0 A0
// PC1 A1
// PC2 A2
// PC3 A3
// PC4 A4
// PC5 A5
// 
// PORTD
// PD0 D0/RX
// PD1 D1/TX
// PD2 D2
// PD3 D3
// PD4 D4
// PD5 D5
// PD6 D6
// PD7 D7
//
// PORTB
// PB0 D8
// PB1 D9
// PB2 D10
// PB3 D11/MOSI
// PB4 D12/MISO
// PB5 D13/SCK
//
#define PB4     4 //D12
#define PB3     3 //D11
#define PB2     2 //D10
//
#define PD5     5 //D5
#define PD4     4 //D4
#define PD3     3 //D3
//
// 
// #define TM_OUT          PORTD
// #define TM_IN           PIND
// #define TM_DDR          DDRD
// #define TM_BIT_RCK      _BV(PD3)
// #define TM_BIT_SCK      _BV(PD4)
// #define TM_BIT_DIO      _BV(PD5)
//
#define TM_OUT          PORTB
#define TM_IN           PINB
#define TM_DDR          DDRB
#define TM_BIT_RCK      _BV(PB2)
#define TM_BIT_SCK      _BV(PB3)
#define TM_BIT_DIO      _BV(PB4)
//

// ----------------------------------------------------------------------------
// Functions and parameters
// ----------------------------------------------------------------------------

// Parameters for setNumber()
#define TM_RIGHT        0x01
#define TM_LEFT         0x00

// Parameters for setNumberPad()
#define TM_PAD_SPACE    0xFF
#define TM_PAD_0        0x03

// Class to control a 8x 7-segment, 
// powered by two 74HC595 chip set.
class HC595
{
public:

    // Construct a 74HC595 object and initialize all the pins/ports used.
    HC595();


    // Clear the 7-segment displays
    void clear();

    // Set a single 7-segment display to the given byte value.
    // This allows direct control of the elements to do spinning animations etc.
    void setByte(const uint8_t pos, const uint8_t value);

    // Display a single digit at the given position.
    // Position is left-to-right, starting with 0.
    void setDigit(const uint8_t pos, const uint8_t value);

    // Display an unsigned number at a given offset and alignment.
    void setNumber(uint32_t number, uint8_t offset = 7, const uint8_t align = TM_RIGHT);

    // Display an unsigned number at a given offset and pad it with 0's or
    // spaces to a desired with. This function is helpful when the numbers can
    // fluctuate in length (ex. 100, 5, 1000) to avoid flickering segments.
    void setNumberPad(uint32_t number, uint8_t offset, uint8_t width, const uint8_t pad = TM_PAD_SPACE);

    // Display an unsigned number in hex format at a given offset and pad it
    // with 0's or spaces to a desired with.
    void setNumberHex(uint32_t number, uint8_t offset, uint8_t width, const uint8_t pad = TM_PAD_SPACE);

    // Draw a character at a given position.
    // Not all characters are supported, check HC595Font.h for an overview.
    void setChar(const uint8_t pos, const char value);

    // Display a string starting at a given offset.
    void setChars(const char* value, uint8_t offset = 0);

    // Set which "dots" should be enabled.
    // Mask is mapped right to left (ex. 0x01 = right-most dot)
    void setDots(const uint8_t mask);

protected:
    uint8_t _dotMask;
};

#endif
