//
// derived from AVR TM1638 "Library" v1.01
//
#ifndef HC595FONT_H
#define HC595FONT_H

#include <avr/pgmspace.h>

#define TM_DOT          0B11111110

#define TM_MINUS        0B11111101
#define TM_PLUS         0B10011101
#define TM_BLANK        0B11111111
#define TM_DEGREES      0B00111011
#define TM_UNDERSCORE   0B11101111
#define TM_EQUALS       0B11101101
#define TM_CHAR_ERR     0B10110110
#define TM_LEFT         0B01100011
#define TM_RIGHT        0B00001111

uint8_t inline HC595_map_char(const char ch)
{
    uint8_t rc = 0;

    switch (ch)
    {
        case '-':
            rc = TM_MINUS;
            break;

        case '+':
            rc = TM_PLUS;
            break;

        case ' ':
            rc = TM_BLANK;
            break;

        case '^':
            rc = TM_DEGREES;
            break;

        case '_':
            rc = TM_UNDERSCORE;
            break;

        case '=':
            rc = TM_EQUALS;
            break;

        case '(':
            rc = TM_LEFT;
            break;

        case ')':
            rc = TM_RIGHT;
            break;

        default:
            break;
    }

    return rc;
}

//      Bits:                 Hex:
//        -- 0 --               -- 01 --
//       |       |             |        |
//       5       1            20        02
//       |       |             |        |
//        -- 6 --               -- 40 --
//       |       |             |        |
//       4       2            10        04
//       |       |             |        |
//        -- 3 --  .7           -- 08 --   .80
//    0B01234567 bit position
//    0 = on
//    1 = off


PROGMEM const uint8_t TM_DIGITS[] =
{
    0B00000011, // 0 0x03  
    0B10011111, // 1 0xCF
    0B00100101, // 2
    0B00001101, // 3
    0B10011001, // 4
    0B01001001, // 5
    0B01000001, // 6
    0B00011111, // 7
    0B00000001, // 8
    0B00001001, // 9

    0B00010001, // A
    0B11000001, // b
    0B01100011, // C
    0B10000101, // d
    0B01100001, // E
    0B01110001, // F

    // HEX DIGITS END

    0B01000001, // G
    0B10010001, // H
    0B10011111, // I
    0B10000111, // J
    0B10001001, // K (same as H)
    0B11100011, // L
    0B00010011, // M
    0B11010101, // n
    0B00000011, // O
    0B00110001, // P
    0B00100001, // Q
    0B11110101, // r
    0B01001001, // S
    0B11100001, // t
    0B11000111, // U
    0B10000011, // V
    0B10000001, // W
    0B10001001, // X (same as H)
    0B10011001, // Y
    0B00100101  // Z    
};

#endif
